@@ -9,7 +9,7 @@ a simple API through which apps can find, fetch and install updates, and a
 bootstrapping mechanism that keeps the app safe in the face of failed or
 partial updates.
 
-Esky is currently capable of freezing apps with bbfreeze, cxfreeze py2exe and
+Esky is currently capable of freezing apps with bbfreeze, cxfreeze, py2exe and
 py2app. Adding support for other freezer programs should be straightforward;
 patches will be gratefully accepted.
 
@@ -92,7 +92,7 @@ from __future__ import with_statement
 
 __ver_major__ = 0
 __ver_minor__ = 5
-__ver_patch__ = 0
+__ver_patch__ = 1
 __ver_sub__ = ""
 __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,__ver_patch__,__ver_sub__)
 
@@ -276,10 +276,8 @@ class Esky(object):
                 (_,v,_) = split_app_version(new_version)
                 self.install_version(v)
                 best_version = new_version
-            #  Now we can safely remove anything that's not part of the
-            #  best version's bootstrap env.  Exceptions are the best
-            #  version itself, the currently-executing version, and any
-            #  locked versions.
+            #  Now we can safely remove all the old versions.
+            #  We except the current-executing version and any locked versions.
             manifest = self._version_manifest(best_version)
             manifest.add("updates")
             manifest.add("locked")
@@ -298,8 +296,6 @@ class Esky(object):
                             pass
                         else:
                             self._try_remove(appdir,nm,manifest)
-                    else:
-                        self._try_remove(appdir,nm,manifest)
             if self.version_finder is not None:
                 self.version_finder.cleanup(self)
         finally:
@@ -416,37 +412,46 @@ class Esky(object):
         """Uninstall the specified version of the app."""
         target_name = join_app_version(self.name,version,self.platform)
         target = os.path.join(self.appdir,target_name)
+        bsfile = os.path.join(target,"esky-bootstrap.txt")
         self.lock()
         try:
             if not os.path.exists(target):
                 return
             #  Clean up the bootstrapping environment in a transaction.
-            trn = FSTransaction()
+            #  This might fail on windows if the version is locked.
             try:
-                #  Get set of all files that must stay in the main appdir
-                to_keep = set()
-                for vname in os.listdir(self.appdir):
-                    if vname == target_name:
-                        continue
-                    details = split_app_version(vname)
-                    if details[0] != self.name:
-                        continue
-                    if parse_version(details[1]) < parse_version(version):
-                        continue
-                    to_keep.update(self._version_manifest(vname))
-                #  Remove files used only by the version being removed
-                to_rem = self._version_manifest(target_name) - to_keep
-                for nm in to_rem:
-                    fullnm = os.path.join(self.appdir,nm)
-                    trn.remove(fullnm)
-            except Exception:
-                trn.abort()
-                raise
-            else:
-                trn.commit()
+                trn = FSTransaction()
+                try:
+                    #  Get set of all files that must stay in the main appdir
+                    to_keep = set()
+                    for vname in os.listdir(self.appdir):
+                        if vname == target_name:
+                            continue
+                        details = split_app_version(vname)
+                        if details[0] != self.name:
+                            continue
+                        if parse_version(details[1]) < parse_version(version):
+                            continue
+                        to_keep.update(self._version_manifest(vname))
+                    #  Remove files used only by the version being removed
+                    to_rem = self._version_manifest(target_name) - to_keep
+                    for nm in to_rem:
+                        fullnm = os.path.join(self.appdir,nm)
+                        trn.remove(fullnm)
+                except Exception:
+                    trn.abort()
+                    raise
+                else:
+                    trn.commit()
+            except EnvironmentError:
+                try:
+                    open(bsfile,"a").close()
+                except EnvironmentError:
+                    raise VersionLockedError("version in use: %s" % (version,))
+                else:
+                    raise
             #  Disable the version by removing its esky-bootstrap.txt file.
             #  To avoid clobbering in-use version, respect locks on this file.
-            bsfile = os.path.join(target,"esky-bootstrap.txt")
             if sys.platform == "win32":
                 try:
                     os.unlink(bsfile)
@@ -456,7 +461,11 @@ class Esky(object):
                 f = open(bsfile,"r+")
                 try:
                     fcntl.lockf(f.fileno(),fcntl.LOCK_EX|fcntl.LOCK_NB)
-                except OSError:
+                except EnvironmentError, e:
+                    if not e.errno:
+                        raise
+                    if e.errno not in (errno.EACCES,errno.EAGAIN,):
+                        raise
                     raise VersionLockedError("version in use: %s" % (version,))
                 else:
                     os.unlink(bsfile)
